package com.diagbot.config.security;

import com.diagbot.facade.TokenFacade;
import com.diagbot.util.HttpUtils;
import com.diagbot.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDecisionManager;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.authentication.AccountExpiredException;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;

/**
 * @Description: 自定义权限拦截
 * @author: gaodm
 * @time: 2018/8/23 13:46
 */
@Service
public class UrlAccessDecisionManager implements AccessDecisionManager {
    @Autowired
    private TokenFacade tokenFacade;

    @Override
    public void decide(Authentication authentication, Object object, Collection<ConfigAttribute> configAttributes) throws AccessDeniedException, InsufficientAuthenticationException {
        HttpServletRequest request = ((FilterInvocation) object).getHttpRequest();
        String url, method;
        if (matchPermitAllUrl(request)) {
            return;
        }
        if ("anonymousUser".equals(authentication.getPrincipal())) {
            throw new AccessDeniedException("no right");
        } else {
            String tokenStr = HttpUtils.getHeaders(request).get("Authorization");
            if (StringUtil.isNotEmpty(tokenStr)) {
                tokenStr = tokenStr.replaceFirst("Bearer ", "");
                Boolean res = tokenFacade.verifyToken(tokenStr, 1);
                if (!res) {
                    throw new AccountExpiredException("token expire");
                }
            }
            for (GrantedAuthority ga : authentication.getAuthorities()) {
                String[] authority = ga.getAuthority().split(";");
                url = authority[0];
                method = authority[1];
                if (matchers(url, request)) {
                    if (method.equals(request.getMethod()) || "ALL".equals(method)) {
                        return;
                    }
                }
            }
        }
        throw new AccessDeniedException("no right");
    }


    @Override
    public boolean supports(ConfigAttribute attribute) {
        return true;
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return true;
    }

    private Boolean matchPermitAllUrl(HttpServletRequest request) {
        if (matchers("/swagger/**", request)
                || matchers("/v2/**", request)
                || matchers("/swagger-ui.html/**", request)
                || matchers("/swagger-resources/**", request)
                || matchers("/webjars/**", request)
                || matchers("/druid/**", request)
                || matchers("/actuator/**", request)
                || matchers("/hystrix/**", request)
                || matchers("/sys/user/getJwt", request)
                || matchers("/sys/user/getJwtNoPass", request)
                || matchers("/sys/user/refreshJwt", request)
                || matchers("/sys/dictionaryInfo/getDictionary", request)
                || matchers("/sys/user/checkToken", request)
                || matchers("/oauth/token", request)
                || matchers("/oauth/check_token", request)
                || matchers("/cache/clear", request)
                || matchers("/qc/behospitalInfo/execule", request)
                || matchers("/qc/behospitalInfo/analyze_rpc", request)
                || matchers("/qc/behospitalInfo/analyze_api", request)
                || matchers("/qc/behospitalInfo/analyze_run", request)
                || matchers("/qc/module/getById", request)
                || matchers("/qc/module/getModuleMap", request)
                || matchers("/qc/cases/getQcCases", request)
                || matchers("/qc/behospitalInfo/page", request)
                || matchers("/qc/casesEntryHospital/getQcCasesEntryAll", request)
                || matchers("/qc/casesEntryHospital/getQcCasesAll", request)
                || matchers("/qc/behospitalInfo/getByBehospitalCode", request)
                || matchers("/bas/dept/getList", request)
                || matchers("/bas/dept/getListUser", request)
                || matchers("/qc/behospitalInfo/page_dept", request)
                || matchers("/qc/behospitalInfo/page_person", request)
                || matchers("/qc/behospitalInfo/page_group", request)
                || matchers("/qc/casesEntryHospital/findQcCasesEntry", request)
                || matchers("/qc/behospitalInfo/exportExcel", request)
                || matchers("/qc/behospitalInfo/exportQcresult", request)
                || matchers("/qc/behospitalInfo/exportQcresultByDept", request)
                || matchers("/qc/behospitalInfo/exportQcresultByGroup", request)
                || matchers("/qc/abnormal/getQcAnnormalMode", request)
                || matchers("/qc/dataimport/import", request)
                || matchers("/qc/dataimport/dataimportPrepare", request)
                || matchers("/qc/dataimport/test", request)
                || matchers("/sys/user/pageset/getPageSet", request)
                || matchers("/sys/user/pageset/savePageSet", request)
                || matchers("/consoleByDept/getDept", request)
                || matchers("/console/entryRejectPercent", request)
                || matchers("/console/qcResultLevelPercent", request)
                || matchers("/console/averageStatistics", request)
                || matchers("/console/entryByDept", request)
                || matchers("/console/entryCountGroupByCase", request)
                || matchers("/console/entryCountGroupByCasePage", request)
                || matchers("/console/entryCountGroupByEntry", request)
                || matchers("/console/entryCountGroupByEntryPage", request)
                || matchers("/console/entryGroupByEntryInnerPage", request)
                || matchers("/console/getAverageDayNum", request)
                || matchers("/console/getAverageDayNumPage", request)
                || matchers("/console/getAverageFee", request)
                || matchers("/console/getAverageFeePage", request)
                || matchers("/console/getAverageScore", request)
                || matchers("/console/getAverageScoreByDeptClass", request)
                || matchers("/console/getAverageScoreByDeptPage", request)
                || matchers("/console/getLevelResultDept", request)
                || matchers("/console/homePageLevelLimit", request)
                || matchers("/console/homePageLevelStatistics", request)
                || matchers("/console/leaveHosCount", request)
                || matchers("/console/levelPercentGroupByDeptPage", request)
                || matchers("/console/levelStatistics", request)
                || matchers("/console/levelStatisticsByDeptClass", request)
                || matchers("/console/mrCount", request)
                || matchers("/console/mrStatistics", request)
                || matchers("/console/qcResultShortPage", request)
                || matchers("/console/resultStatistics", request)
                || matchers("/console/resultStatisticsByDeptPage", request)
                || matchers("/consoleByDept/entryCountGroupByCaseAndDept", request)
                || matchers("/consoleByDept/entryCountGroupByCaseAndDeptPage", request)
                || matchers("/consoleByDept/entryCountGroupByEntryAndDept", request)
                || matchers("/consoleByDept/entryCountGroupByEntryAndDeptPage", request)
                || matchers("/consoleByDept/entryGroupByEntryAndDeptInnerPage", request)
                || matchers("/consoleByDept/homePageLevelByDeptLimit", request)
                || matchers("/consoleByDept/homePageLevelStatisticsByDept", request)
                || matchers("/consoleByDept/leaveHosCountByDept", request)
                || matchers("/consoleByDept/levelStatisticsByDept", request)
                || matchers("/consoleByDept/mrCountByDept", request)
                || matchers("/consoleByDept/qcResultShortByDeptPage", request)
                || matchers("/consoleByDept/resultStatisticsByDeptAndDoctorPage", request)
                || matchers("/console/export/homePageLevelExport", request)
                || matchers("/console/export/entryGroupByEntryExport", request)
                || matchers("/console/export/levelExport", request)
                || matchers("/console/entryStatistics", request)
                || matchers("/console/export/levelExport_TZ", request)
                || matchers("/console/export/getAverageDayNumExport", request)
                || matchers("/console/export/getAverageFeeExport", request)
                || matchers("/console/export/levelPercentGroupByDeptExport", request)
                || matchers("/console/export/entryCountGroupByEntryExport", request)
                || matchers("/console/export/entryCountGroupByCaseExport", request)
                || matchers("/console/export/entryStatisticsExport", request)
                || matchers("/console/export/qcResultShortPageExport", request)
                || matchers("/qc/data/sendDoctorInfos", request)
                || matchers("/qc/data/sendDeptInfos", request)
                || matchers("/qc/data/sendRecordTypes", request)
                || matchers("/qc/data/sendMrRecordIng", request)
                || matchers("/qc/data/sendMrContent", request)
                || matchers("/qc/data/sendMrRecord", request)
                || matchers("/qc/data/sendPatientInfo", request)
                || matchers("/qc/data/sendDoctorAdvice", request)
                || matchers("/qc/data/sendHomePageIng", request)
                || matchers("/qc/data/sendHomePage", request)
                || matchers("/qc/data/sendHomeDiagnose", request)
                || matchers("/qc/data/sendHomeOperation", request)
                || matchers("/qc/data/sendCrisis", request)
                || matchers("/", request)) {
            return true;
        }
        return false;
    }

    private boolean matchers(String url, HttpServletRequest request) {
        AntPathRequestMatcher matcher = new AntPathRequestMatcher(url);
        if (matcher.matches(request)) {
            return true;
        }
        return false;
    }
}